/*******************************************************************************
 * Copyright (C) Altimetrik 2018. All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Altimetrik. You shall not disclose such Confidential Information
 * and shall use it only in accordance with the terms and conditions
 * entered into with Altimetrik.
 ******************************************************************************/
package com.altimetrik.playground.innovation.webmvc;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;

import javax.servlet.ReadListener;
import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import org.apache.commons.io.IOUtils;

public class RequestWrapper extends HttpServletRequestWrapper {

	private ByteArrayOutputStream cachedInputStream;

	public RequestWrapper(HttpServletRequest request) {
		super(request);
	}

	@Override
	public ServletInputStream getInputStream() throws IOException {
		if (this.cachedInputStream == null) {
			// Cache the request input stream.
			this.cacheInputStream();
		}
		// An input stream which reads the cached input stream.
		return new CachedServletInputStream();
	}

	@Override
	public BufferedReader getReader() throws IOException {
		return new BufferedReader(new InputStreamReader(this.getInputStream()));
	}

	private void cacheInputStream() throws IOException {
		// Cache the input stream in order to read it multiple times.
		this.cachedInputStream = new ByteArrayOutputStream();
		IOUtils.copy(super.getInputStream(), this.cachedInputStream);
	}

	public class CachedServletInputStream extends ServletInputStream {

		private ByteArrayInputStream input;

		public CachedServletInputStream() {
			// Create a new input stream from the cached input stream.
			this.input = new ByteArrayInputStream(cachedInputStream.toByteArray());
		}

		@Override
		public int read() throws IOException {
			return this.input.read();
		}

		@Override
		public boolean isFinished() {
			return this.input.available() == 0;
		}

		@Override
		public boolean isReady() {
			return true;
		}

		@Override
		public void setReadListener(ReadListener listener) {
			throw new RuntimeException("No async processing implementation");
		}

	}

}
